import requests

from typing import List, Tuple

from .__user_config import UserConfiguration


class Client:
    __base_url: str = None
    __headers: dict

    def __init__(self, base_url: str, user_id: str, roles: List):
        self.__base_url = base_url
        self.__headers = {
            'x-user-id': user_id,
            'x-user-roles': ','.join(roles)
        }

    @staticmethod
    def from_config(config: UserConfiguration) -> 'Client':
        return Client(config.url(), config.user(), config.roles())

    def command(self, cmd: str, args: dict = None, headers: dict = None) -> Tuple[int,dict]:
        request_body = { 'command': cmd }
        if args:
            request_body.update(args)
        if headers:
            headers = {**self.__headers,**headers}
        else:
            headers = self.__headers
        response = requests.post(self.__base_url + 'commands', json = request_body, headers = headers)
        if response.status_code < 200 or response.status_code > 299:

            raise RuntimeError("call to Maquette controller was not successful ¯\\_(ツ)_/¯\n"
                               "status code: " + str(response.status_code) + ", content:\n" + response.text)
        else:
            if ("Accept", "application/csv") or ("Accept", "text/plain") in headers.items():
                result = response.content
            else:
                result = response.json()
            return response.status_code, result

    def get(self, url: str, headers = None) -> requests.Response:
        if headers:
            headers = {**self.__headers,**headers}
        else:
            headers = self.__headers
        response = requests.get(self.__base_url + url, headers = headers)
        if response.status_code < 200 or response.status_code > 299:
            raise RuntimeError("call to Maquette controller was not successful ¯\\_(ツ)_/¯\n"
                               "status code: " + str(response.status_code) + ", content:\n" + response.text)
        else:
            return response

    def put(self, url: str, json = None, files = None, headers = None) -> requests.Response:
        if headers:
            headers = {**self.__headers,**headers}
        else:
            headers = self.__headers
        response = requests.put(self.__base_url + url, json = json, files = files, headers = headers)
        if response.status_code < 200 or response.status_code > 299:
            raise RuntimeError("call to Maquette controller was not successful ¯\\_(ツ)_/¯\n"
                               "status code: " + str(response.status_code) + ", content:\n" + response.text)
        else:
            return response

    def post(self, url: str, json = None, files = None, headers = None) -> requests.Response:
        if headers:
            headers = {**self.__headers,**headers}
        else:
            headers = self.__headers
        response = requests.post(self.__base_url + url, json = json, files = files, headers = headers)
        if response.status_code < 200 or response.status_code > 299:
            raise RuntimeError("call to Maquette controller was not successful ¯\\_(ツ)_/¯\n"
                               "status code: " + str(response.status_code) + ", content:\n" + response.text)
        else:
            return response


import os

class UserConfiguration:

    __mq_home: str = None

    def __init__(self, mq_home: str):
        self.__mq_home = mq_home

    def url(self) -> str:
        return os.environ.get('MQ_BASE_URL','http://localhost:9042/api/')

    def user(self) -> str:
        return os.environ.get('MQ_USERNAME', 'alice')

    def roles(self) -> str:
        return os.environ.get('MQ_ROLES', 'a-team, b-team').split(", ")


from maquette.__mq import EProjectPrivilege, EDatasetPrivilege, EAuthorizationType, EDataClassification, EDataVisibility, EPersonalInformation
import maquette as mq
import pandas as pd

### Create Project
mq.project(name="just-a-sample-project", title="Sample Title", summary="This is a summary").create()

### Show a list of all projects
print(mq.projects(to_csv=True))

### add a dataset to your current project
#mq.dataset('another-dataset').create()

### add a dataset to a specific project
#mq.project(name="just-a-sample-project").dataset('another-dataset').create()

### show all datasets of a specific project
#print(mq.project(name="just-a-sample-project").datasets())

# show all datasets of the current project
#print(mq.datasets())

### upload data to the dataset
#testdf = pd.DataFrame({'col1': [1, 2], 'col2': [3, 4]})
#print(mq.dataset('another-dataset').put(testdf, "muahahaha"))

### Get a Dataframe from a Dataset
#df = mq.dataset('another-dataset').get()
#print(df)

### delete the two datasets from above
# mq.dataset('another-dataset').remove()
# mq.project(name="just-a-sample-project").dataset('another-dataset').delete()

# TODO Collection testen
### Create a new collection
# mq.project("just-a-sample-project").collection('another-collection').create()

### Put a file into the collection
# ifile = open('maquette/sample/just-a.csv', 'rb')
# mq.project("just-a-sample-project").collection('another-collection').put(data=ifile, short_description="just a file")

# TODO Data Source testen
### Create a Data Source

# TODO Streams testen

### Delete the project in the end
print(mq.project(name="just-a-sample-project").remove())
